@@ -141,7 +141,7 @@ static void
 setup_gsettings (NaTrayApplet *applet)
 {
   applet->priv->settings = mate_panel_applet_settings_new (MATE_PANEL_APPLET (applet), NA_TRAY_SCHEMA);
-  g_signal_connect (applet->priv->settings, "changed::" KEY_MIN_ICON_SIZE, G_CALLBACK (gsettings_changed_min_icon_size), applet);
+  g_signal_connect_object (G_OBJECT(applet->priv->settings), "changed::" KEY_MIN_ICON_SIZE, G_CALLBACK (gsettings_changed_min_icon_size), applet, 0);
 }
 
 static void
@@ -194,7 +194,7 @@ ensure_prefs_window_is_created (NaTrayApplet *applet)
   gtk_spin_button_set_range (GTK_SPIN_BUTTON (applet->priv->dialog->min_icon_size_spin), 7, 130);
   gtk_spin_button_set_value (GTK_SPIN_BUTTON (applet->priv->dialog->min_icon_size_spin), applet->priv->min_icon_size);
 
-  g_signal_connect_swapped (applet->priv->dialog->min_icon_size_spin, "value_changed",
+  g_signal_connect_swapped (applet->priv->dialog->min_icon_size_spin, "value-changed",
                             G_CALLBACK (na_preferences_dialog_min_icon_size_changed),
                             applet);
 
@@ -276,11 +276,11 @@ static void about_cb(GtkAction* action, NaTrayApplet* applet)
 		"program-name", _("Notification Area"),
 		"title", _("About Notification Area"),
 		"authors", authors,
-		//"comments", _(comments),
+		/* "comments", _(comments), */
 		"copyright", _("Copyright \xc2\xa9 2002 Red Hat, Inc.\n"
 		               "Copyright \xc2\xa9 2003-2006 Vincent Untz\n"
 		               "Copyright \xc2\xa9 2011 Perberos\n"
-		               "Copyright \xc2\xa9 2012-2020 MATE developers"),
+		               "Copyright \xc2\xa9 2012-2021 MATE developers"),
 		"documenters", documenters,
 		"logo-icon-name", NOTIFICATION_AREA_ICON,
 		"translator-credits", _("translator-credits"),
@@ -320,7 +320,7 @@ na_tray_applet_realize (GtkWidget *widget)
 
   setup_gsettings (applet);
 
-  // load min icon size
+  /* load min icon size */
   gsettings_changed_min_icon_size (applet->priv->settings, KEY_MIN_ICON_SIZE, applet);
 
   applet->priv->builder = gtk_builder_new ();
