@@ -15,12 +15,12 @@
  * along with this program. If not, see <http://www.gnu.org/licenses/>.
  */
 
-#include "config.h"
+#include <config.h>
 
 #include "sn-dbus-menu-item.h"
 
 static GdkPixbuf *
-pxibuf_new (GVariant *variant)
+pixbuf_new (GVariant *variant)
 {
   gsize length;
   const guchar *data;
@@ -149,7 +149,7 @@ sn_dbus_menu_item_new (GVariant *props)
       else if (g_strcmp0 (prop, "icon-name") == 0)
         item->icon_name = g_variant_dup_string (value, NULL);
       else if (g_strcmp0 (prop, "icon-data") == 0)
-        item->icon_data = pxibuf_new (value);
+        item->icon_data = pixbuf_new (value);
       else if (g_strcmp0 (prop, "label") == 0)
         item->label = g_variant_dup_string (value, NULL);
       else if (g_strcmp0 (prop, "shortcut") == 0)
@@ -252,7 +252,7 @@ sn_dbus_menu_item_new (GVariant *props)
 }
 
 void
-sn_dubs_menu_item_free (gpointer data)
+sn_dbus_menu_item_free (gpointer data)
 {
   SnDBusMenuItem *item;
 
@@ -333,7 +333,7 @@ sn_dbus_menu_item_update_props (SnDBusMenuItem *item,
           GtkWidget *image;
 
           g_clear_object (&item->icon_data);
-          item->icon_data = pxibuf_new (value);
+          item->icon_data = pixbuf_new (value);
 
           if (item->icon_data)
             {
